﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Implementation;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CategoryHelperRepository : ICategoryHelperRepository
    {
        #region Private
        private string _tableName;
        private string AllFields;
        private string AllLocaleFields;
        private string AllFieldsWithLocale;
        private UrlNameAccessHelper _urlNameHelper;

        private string ConvertFieldName( string fieldName )
        {
            string field = _tableName.ToLower() + "id";
            if (fieldName.ToLower() == field)
            {
                return _tableName + "." + _tableName + "ID";
            }
            else
            {
                return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            string field = _tableName.ToLower() + "id";
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( field ))
                return "ORDER BY " + _tableName + "." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY L." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string CreateSortByTextGetByCategoryDepartment( string sortBy )
        {
            string field = _tableName.ToLower() + "id";
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( field ))
                return "ORDER BY " + _tableName + "." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return "ORDER BY " + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return "ORDER BY C." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string CreateSortByTextForProduct( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "productid" ))
                return " ORDER BY Product." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "name" ))
                return " ORDER BY L." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "sortorder" ))
                return " ORDER BY C." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "price" ))
                return " ORDER BY PP." + sortBy.Trim() + " ";
            else
                return " ORDER BY " + sortBy;
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private void DeleteLocales( string ID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM " + _tableName + "Locale " +
                "WHERE " + _tableName + "ID = @ID; ",
                DataAccess.CreateParameterString( ID ) );
        }

        private string GenerateQueryStringGetByCategoryDepartmentID(
            BoolFilter isEnabled )
        {
            string query = "SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
            "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, " +
            "    ShippingCost, Manufacturer, Keywords, PP.Price, " +
            "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
            "    RelatedProducts, MerchantRating, " +
            "    Product.DiscountGroupID As ProductDiscountGroupID, " +
            "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
            "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
            "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
            "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, IsProductKit, " +
                // Edit for Recurring.
            "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
            "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                //"    Product" + _tableName + ".SortOrder " + 
            "    C.SortOrder " +
            "    FROM " +
            "    ((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
            "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
            "    LEFT JOIN " +
            "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
            "     ON Product.ProductID = L.ProductID) " +
            "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
            "    LEFT JOIN Product" + _tableName + " C ON Product.ProductID = C.ProductID ) " +
            "    LEFT JOIN " + _tableName + " D ON C." + _tableName + "ID = D." + _tableName + "ID ) " +
            "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
            "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
            "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
            "WHERE C." + _tableName + "ID = @ID " +
            FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) );

            return query;
        }

        #endregion

        #region Public Constructor
        public CategoryHelperRepository( string tableName )
        {
            _tableName = tableName;
            _urlNameHelper = new UrlNameAccessHelper( _tableName + "Locale", _tableName + "ID" );

            AllFields = _tableName + "." + _tableName + "ID, ImageFile, Parent" + _tableName + "ID, " + _tableName + ".IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, IsAnchor, IsShowNewArrival, NewArrivalAmount, " +
            _tableName + "ListLayoutPath, ProductListLayoutPath, RootID ";

            AllLocaleFields = _tableName + "ID, CultureID, Name, Description, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, UrlName ";

            AllFieldsWithLocale = _tableName + "." + _tableName + "ID, Name, Description, ImageFile, Parent" + _tableName + "ID, UrlName, " + _tableName + ".IsEnabled, " +
            "SortOrder, DiscountGroupID, Other1, Other2, Other3, Other4, Other5, IsAnchor, IsShowNewArrival, NewArrivalAmount, " +
            _tableName + "ListLayoutPath, ProductListLayoutPath, RootID, MetaKeyword, MetaDescription, ImageAlt, ImageTitle ";
        }
        #endregion

        #region Public Methods

        public void Delete( string ID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE " +
                    "FROM " + _tableName +
                    " WHERE " + _tableName + "ID = @ID; ",
                    DataAccess.CreateParameterString( ID )
                    );
                DeleteLocales( ID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM " + _tableName + "Locale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID )
            );
        }

        public void UpdateSortOrder( string[] sortedIDs )
        {
            int sortCount = 0;
            foreach (string items in sortedIDs)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE " + _tableName + " SET SortOrder = @SortOrder WHERE " + _tableName + "ID = @ID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( items ) );
            }
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + _tableName + "." + _tableName + "ID, Name, Description, ImageFile, Parent" + _tableName + "ID, UrlName, IsEnabled, " +
                " Other1, Other2, Other3, Other4, Other5, MetaKeyword, MetaDescription, ImageAlt, ImageTitle, IsAnchor " +
                "FROM " + _tableName + " LEFT JOIN " + _tableName + "Locale L " +
                "    ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " + "WHERE 0 = 1; " );

            DataColumn[] keys = new DataColumn[] { table.Columns[_tableName + "ID"] };
            table.PrimaryKey = keys;
            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public bool IsLeaf( string parentID )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM " + _tableName +
                " WHERE Parent" + _tableName + "ID = @ParentID; ",
                DataAccess.CreateParameterString( parentID ) );

            return int.Parse( count ) == 0;
        }

        public bool IsNotLeaf( string parentID )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM " + _tableName +
                " WHERE Parent" + _tableName + "ID = @ParentID; ",
                DataAccess.CreateParameterString( parentID ) );

            return int.Parse( count ) > 0;
        }

        public bool IsUrlNameNotLeaf( string parentUrlName )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " +
                "FROM (" + _tableName +
                " INNER JOIN " + _tableName + " AS P ON " + _tableName + ".Parent" + _tableName + "ID = P." + _tableName + "ID) " +
                " INNER JOIN " + _tableName + "Locale L ON " + "P." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE L.UrlName = @ParentName; ",
                DataAccess.CreateParameterString( parentUrlName ) );
            return int.Parse( count ) > 0;
        }

        public void RefreshUrlName( string ID, string name )
        {
            if (!string.IsNullOrEmpty( name ))
                _urlNameHelper.UpdateUrlName( ID, name );
        }

        public bool HasItInDiscountGroup( string discountgroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM " + _tableName + " " +
                "WHERE DiscountGroupID = @DiscountGroupID;",
                DataAccess.CreateParameterString( discountgroupID ) );

            if (table.Rows.Count == 0)
                return false;
            else
                return true;
        }

        public DataTable GetOne( Culture culture, string categoryID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName +
                " LEFT JOIN  " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L  " +
                "       ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE " + _tableName + "." + _tableName + "ID = @ID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( categoryID ) );

            return table;
        }

        public DataTable GetOneByUrlName( Culture culture, string urlName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN  " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L  " +
                "       ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( urlName ) );

            return table;
        }

        public DataTable GetByParentID(
                Culture culture,
                string parentID,
                string sortBy,
                BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE Parent" + _tableName + "ID = @ParentCategoryID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( parentID ) );

            return table;
        }

        public DataTable GetByParentIDAndRootID(
            Culture culture,
            string parentID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE Parent" + _tableName + "ID = @ParentCategoryID AND RootID = @RootID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( parentID ),
                DataAccess.CreateParameterString( rootID ) );

            return table;
        }

        public DataTable GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE RootID = @RootID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( rootID ) );

            return table;
        }

        public DataTable GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            string sqlCommand = "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE RootID = @RootID " +
                "AND " + _tableName + "." + _tableName + "ID NOT IN (SELECT C.Parent" + _tableName + "ID AS " + _tableName + "ID " +
                "FROM " + _tableName + " AS C " +
                "Group by C.Parent" + _tableName + "ID)" +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ";
            DataTable table = DataAccess.ExecuteSelect(
                sqlCommand,
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( rootID ) );

            return table;
        }

        public DataTable GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            string sqlCommand = "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE RootID = @RootID " +
                "AND " + _tableName + "." + _tableName + "ID IN (SELECT C.Parent" + _tableName + "ID AS " + _tableName + "ID " +
                "FROM " + _tableName + " AS C " +
                "Group by C.Parent" + _tableName + "ID)" +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ";

            DataTable table = DataAccess.ExecuteSelect(
                sqlCommand,
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( rootID ) );

            return table;
        }

        public DataTable GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            string sqlCommand = "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE RootID = @RootID " +
                "AND " + _tableName + "." + _tableName + "ID NOT IN (SELECT distinct C." + _tableName + "ID AS " + _tableName + "ID FROM Product" + _tableName + " AS C) " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ";

            DataTable table = DataAccess.ExecuteSelect(
                sqlCommand,
                DataAccess.CreateParameterString( GetCultureID( culture ) ),
                DataAccess.CreateParameterString( rootID ) );

            return table;
        }

        public DataTable GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE RootID = 0 " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( GetCultureID( culture ) ) );

            return table;
        }

        public DataTable GetByParentUrlName(
            Culture culture,
            string parentName,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex )
        {
            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, * FROM ( " +
                "SELECT DISTINCT C.*, CultureID, Name, Description, ImageAlt, ImageTitle, MetaKeyword, MetaDescription, UrlName " +
                    "FROM " + _tableName + " C INNER JOIN " + _tableName + "Locale L ON C." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE Parent" + _tableName + "ID = ( SELECT " + _tableName + "ID FROM " + _tableName + "Locale WHERE UrlName = @ParentName) AND CultureID = @CultureID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "C.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +

                ") AS Product";

            string extQuery = "with SearchResult as (" + query + ") " +
                " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect( extQuery,
                DataAccess.CreateParameterString( parentName ),
                DataAccess.CreateParameterString( culture.CultureID ) );

            return full;
        }

        public int GetCountByParentUrlName(
            Culture culture,
            string parentName,
            string sortBy,
            BoolFilter isEnabled )
        {
            string query = "with SearchResult as (SELECT ROW_NUMBER() over " +
                "( ORDER BY SortOrder ) as rownum, * FROM " +
                "( SELECT DISTINCT C.*, CultureID, Name, Description, " +
                "ImageAlt, ImageTitle, MetaKeyword, MetaDescription, " +
                "UrlName FROM Category C INNER JOIN CategoryLocale L " +
                "ON C.CategoryID = L.CategoryID WHERE " +
                "ParentCategoryID = ( SELECT CategoryID " +
                "FROM CategoryLocale WHERE UrlName = @ParentName) " +
                "AND CultureID = @CultureID  " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "C.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                " ) " + "AS Product)  " +
                "select COUNT(rownum) from SearchResult";

            string countResult = DataAccess.ExecuteScalar( query,
            DataAccess.CreateParameterString( parentName ),
            DataAccess.CreateParameterString( culture.CultureID ) );

            return Convert.ToInt32( countResult );
        }

        private IList<String> ConvertProductIDTableToList( DataTable table )
        {
            IList<String> list = new List<String>();
            foreach (DataRow row in table.Rows)
                list.Add( row["ProductID"].ToString() );

            return list;
        }

        public IList<String> GetProductIDListByCategoryDepartment(
            string[] categoryIDs,
            string[] departmentIDs,
            BoolFilter isEnabled,
            string storeID )
        {

            ArrayList parameterList = new ArrayList();

            for (int i = 0; i <= categoryIDs.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categoryIDs[i] ) );
            }

            for (int i = 0; i <= departmentIDs.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( departmentIDs[i] ) );
            }

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT DISTINCT pc.ProductID " +
                "FROM Product p, ProductCategory pc, ProductDepartment pd, Category D, Department DD " +
                CreateCategoryDepartmentQuery( categoryIDs, departmentIDs, "D." ),
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return ConvertProductIDTableToList( table );
        }

        public decimal GetMaxPriceByCategoryDepartment(
           Culture culture,
           string[] categoryID,
           string[] departmentID,
           BoolFilter isEnabled,
           string storeID
)
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            string query = "SELECT Max(Price) FROM(" +
                " SELECT C.ProductID, Name, ShortDescription, LongDescription, UrlName, C.Sku, " +
                " C.Model, C.ImageSecondary, SumStock, C.Weight, C.FixedShippingCost, " +
                " C.ShippingCost, C.Manufacturer, C.Keywords,C.Price, " +
                " C.IsTodaySpecial, C.IsDownloadable, C.DownloadPath, Product.IsEnabled, " +
                " C.RelatedProducts, C.MerchantRating, Product.DiscountGroupID As ProductDiscountGroupID, " +
                " C." + _tableName + "DiscountGroupID,     C.IsGiftCertificate, C.IsFixedPrice, C.ManufacturerPartNumber, C.Upc,            " +
                " C.TaxClassID, Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, C.UseInventory, " +
                " C.ProductDetailsLayoutPath, C.IsAffiliate, C.IsCustomPrice, C.IsCallForPrice, Product.Brand, C.ReturnTime, C.IsProductKit, " +
                " C.IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles, " +
                " RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                " C.SortOrder   " +
                " from    " +
                " (Product INNER JOIN   " +

                " ( SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    A.SortOrder " +
                "    FROM " +
                "    ((((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " A ON Product.ProductID = A.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON A." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductDepartment PD ON Product.ProductID = PD.ProductID ) " +
                "    LEFT JOIN Department DD ON PD.DepartmentID = DD.DepartmentID )  " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
               CreateCategoryDepartmentQuery( categoryID, departmentID, "A." ) +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                "       )  " +
                "   as C on Product.ProductID = C.ProductID) " +
                "   where C.ProductID = product.ProductID " +
                " ) AS rownum";
            for (int i = 0; i <= categoryID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categoryID[i] ) );
            }

            for (int i = 0; i <= departmentID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( departmentID[i] ) );
            }

            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            string result = DataAccess.ExecuteScalar( query, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            if (String.IsNullOrEmpty( result ))
            {
                result = "0";
            }

            return Convert.ToDecimal( result );
        }

        public int GetFacetCount(
           Culture culture,
           string[] categoryID,
           string[] departmentID,
           string manufacturerID,
           string minPrice,
           string maxPrice,
           IList<SpecificationItemValue> specValueList,
           string sortBy,
           BoolFilter isEnabled,
           string storeID
)
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            string productTable = "Product.";
            if (specValueList.Count > 0)
                productTable = "PS.";

            string manufacturerQuery = String.Empty;
            if (!String.IsNullOrEmpty( manufacturerID ))
            {
                manufacturerQuery = "   and c.Manufacturer = " + manufacturerID;
            }

            string query = "SELECT COUNT(*) FROM(" +
                " SELECT C.ProductID, Name, ShortDescription, LongDescription, UrlName, C.Sku, " +
                " C.Model, C.ImageSecondary, SumStock, C.Weight, C.FixedShippingCost, " +
                " C.ShippingCost, C.Manufacturer, C.Keywords,C.Price, " +
                " C.IsTodaySpecial, C.IsDownloadable, C.DownloadPath, Product.IsEnabled, " +
                " C.RelatedProducts, C.MerchantRating, Product.DiscountGroupID As ProductDiscountGroupID, " +
                " C." + _tableName + "DiscountGroupID,     C.IsGiftCertificate, C.IsFixedPrice, C.ManufacturerPartNumber, C.Upc,            " +
                " C.TaxClassID, Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, C.UseInventory, " +
                " C.ProductDetailsLayoutPath, C.IsAffiliate, C.IsCustomPrice, C.IsCallForPrice, Product.Brand, C.ReturnTime, C.IsProductKit, " +
                " C.IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles, " +
                " RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                " from    " +
                " (Product INNER JOIN   " +

                " ( SELECT DISTINCT " + productTable + "ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    FROM " +
                "    ((((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " A ON Product.ProductID = A.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON A." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductDepartment PD ON Product.ProductID = PD.ProductID ) " +
                "    LEFT JOIN Department DD ON PD.DepartmentID = DD.DepartmentID ) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                " WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +

                CreateSpecificationCondition( specValueList, "" ) +

               CreateCategoryDepartmentQuery( categoryID, departmentID, "A." ) +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ConditionPrice( minPrice, maxPrice, "PP." ) +
                //+ " "  +
                "       )  " +
                "   as C on Product.ProductID = C.ProductID) " +
                "   where C.ProductID = product.ProductID " +
                manufacturerQuery
                    + ") AS rownum";

            for (int i = 0; i <= specValueList.Count - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( specValueList[i].Value ) );
                parameterList.Add( DataAccess.CreateParameterString( specValueList[i].SpecificationItemID ) );
            }

            for (int i = 0; i <= categoryID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categoryID[i] ) );
            }

            for (int i = 0; i <= departmentID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( departmentID[i] ) );
            }

            if (!String.IsNullOrEmpty( minPrice ))
            {
                parameterList.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( minPrice ) ) );
            }

            if (!String.IsNullOrEmpty( maxPrice ))
            {
                parameterList.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( maxPrice ) ) );
            }

            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            string countResult = DataAccess.ExecuteScalar( query, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return Convert.ToInt32( countResult );
        }

        public DataTable GetAll( Culture culture, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public DataTable GetAllLeafOnly( Culture culture, string sortBy )
        {
            string query = "SELECT " + AllFieldsWithLocale +
                " FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE L." + _tableName + "ID NOT IN  (SELECT C.Parent" + _tableName + "ID AS " + _tableName + "ID" +
                " FROM " + _tableName + " AS C Group by C.Parent" + _tableName + "ID) " +
                CreateSortByText( sortBy ) + "; ";
            DataTable table = DataAccess.ExecuteSelect(
                query,
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public DataTable GetAll( Culture culture, string sortBy, BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                "WHERE 1 = 1 " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public DataTable GetAllWithoutRootCategory( Culture culture, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                " WHERE RootID != 0 " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public DataTable GetLocales( string ID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllLocaleFields +
                "FROM " + _tableName + "Locale " +
                "WHERE " + _tableName + "ID = @ID; ",
                DataAccess.CreateParameterString( ID ) );

            return localeTable;
        }

        public DataTable Search(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
        )
        {
            string categoryFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, categoryFieldName, parameterList );

            string query = "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "     ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ";

            DataTable full = DataAccess.ExecuteSelect( query,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return full;
        }

        public DataTable SearchByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
        )
        {
            string categoryFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( rootID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithAnd( searchFilter, categoryFieldName, parameterList );

            string query = "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "     ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                " Where RootID = @RootID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ";

            DataTable full = DataAccess.ExecuteSelect( query,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return full;
        }

        public DataTable SearchRoot(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
        )
        {
            string departmentFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithAnd( searchFilter, departmentFieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllFieldsWithLocale +
                "FROM " + _tableName + " LEFT JOIN " +
                "     (SELECT * FROM " + _tableName + "Locale WHERE CultureID = @CultureID) AS L " +
                "     ON " + _tableName + "." + _tableName + "ID = L." + _tableName + "ID " +
                " Where RootID = 0 " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return full;
        }

        private string CreateCategoryDepartmentQuery( string[] categoryList, string tablePrefix )
        {
            string query = String.Empty;

            for (int i = 0; i <= categoryList.Length - 1; i++)
            {
                query += tablePrefix + _tableName + "ID = @ID" + i.ToString();
                if (i < categoryList.Length - 1)
                {
                    query += " or ";
                }
            }
            if (!String.IsNullOrEmpty( query ))
            {
                query = " WHERE " + " ( " + query + " ) ";
            }

            return query;
        }

        private string CreateCategoryDepartmentQuery(
           string[] categoryList,
           string[] departmentList,
           string tablePrefix )
        {
            string query = String.Empty;
            string categoryQuery = String.Empty;
            string departmentQuery = String.Empty;

            for (int i = 0; i <= categoryList.Length - 1; i++)
            {
                categoryQuery += tablePrefix + _tableName + "ID = @ID" + i.ToString();
                if (i < categoryList.Length - 1)
                {
                    categoryQuery += " or ";
                }
            }
            if (!String.IsNullOrEmpty( categoryQuery ))
            {
                categoryQuery = " ( " + categoryQuery + " ) and " + "D.IsEnabled = 1 ";
            }

            if (departmentList.Length > 0)
            {
                for (int i = 0; i <= departmentList.Length - 1; i++)
                {
                    departmentQuery += "PD.DepartmentID = @DepID" + i.ToString();
                    if (i < departmentList.Length - 1)
                    {
                        departmentQuery += " or ";
                    }
                }

            }

            if (!String.IsNullOrEmpty( departmentQuery ))
            {
                departmentQuery = " ( " + departmentQuery + " ) and DD.IsEnabled = 1 ";
                if (!String.IsNullOrEmpty( categoryQuery ))
                {
                    query = " WHERE " + "( " + categoryQuery + " and " + departmentQuery + " )";
                }
                else
                {
                    query = " WHERE " + "( " + departmentQuery + " )";
                }
            }
            else
                query = " WHERE " + categoryQuery;

            return query;
        }

        private string CreateSpecificationCondition( IList<SpecificationItemValue> specValueList, string tablePrefix )
        {
            string query = String.Empty;
            string query1 = String.Empty;
            if (specValueList.Count > 0)
            {
                query += " LEFT JOIN (  SELECT ProductID FROM ProductSpecification WHERE";

                for (int i = 0; i <= specValueList.Count - 1; i++)
                {

                    query1 += "( " + tablePrefix + "Value = @Value" + i.ToString() + " and "
                        + tablePrefix + "SpecificationItemID = @SpecificationItemID" + i.ToString() + " ) ";

                    if (i < specValueList.Count - 1)
                    {
                        query1 += " or ";
                    }
                }


                query = query + " ( " + query1 + " ) ";

                query += " GROUP BY ProductID  " +
                  " HAVING Count(ProductID) = " + specValueList.Count + " ) PS ON PS.ProductID = Product.ProductID ";
            }
            return query;
        }

        private string ConditionPrice( string price1, string price2, string prefix )
        {
            if (!String.IsNullOrEmpty( price1 ) && String.IsNullOrEmpty( price2 ))
            {
                return " AND " + prefix + "[Price] >= @Price1 ";
            }
            else if (String.IsNullOrEmpty( price1 ) && !String.IsNullOrEmpty( price2 ))
            {
                return " AND " + prefix + "[Price] <= @Price2 ";
            }
            else if (!String.IsNullOrEmpty( price1 ) && !String.IsNullOrEmpty( price2 ))
            {
                return " AND " + prefix + "[Price] BETWEEN @Price1 AND @Price2 ";
            }
            else
                return "";
        }

        public DataTable GetNewArrivalByCategoryDepartmentID(
            Culture culture,
            string[] categoryID,
            string[] departmentID,
            int limitCount,
            string storeID )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            string productTable = "Product.";

            string query = "SELECT DISTINCT TOP " + limitCount +
                " C.ProductID, Name, ShortDescription,   " +
                " LongDescription, UrlName, C.Sku,        " +
                " C.Model, C.ImageSecondary, SumStock, C.Weight, C.FixedShippingCost, FreeShippingCost,        " +
                " C.ShippingCost, C.Manufacturer,        " +
                " C.Keywords,C.Price,        " +
                " C.IsTodaySpecial, C.IsDownloadable, C.DownloadPath, Product.IsEnabled,        " +
                " C.RelatedProducts, C.MerchantRating,     Product.DiscountGroupID As ProductDiscountGroupID,        " +
                " C.CategoryDiscountGroupID,     C.IsGiftCertificate, C.IsFixedPrice, C.ManufacturerPartNumber, C.Upc,            " +
                " C.TaxClassID,     Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, C.UseInventory,        " +
                " C.ProductDetailsLayoutPath, C.IsAffiliate, C.IsCustomPrice, C.IsCallForPrice, Product.Brand, C.ReturnTime, C.IsProductKit,        " +
                " C.IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles,       " +
                " RecurringNumberOfTrialCycles, RecurringTrialAmount,C.CreateDate      " +
                " FROM    " +
                " (Product INNER JOIN   " +

                " ( SELECT DISTINCT " + productTable + "ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost" +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit,CreateDate, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    FROM " +
                "    ((((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " A ON Product.ProductID = A.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON A." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductDepartment PD ON Product.ProductID = PD.ProductID ) " +
                "    LEFT JOIN Department DD ON PD.DepartmentID = DD.DepartmentID )  " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                " WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +

                CreateCategoryDepartmentQuery( categoryID, departmentID, "A." ) +
                    FlagFilterHelper.CreateFlagFilterWithAnd( "Product.IsEnabled", FlagFilter.ShowTrue ) +
                "       )  " +
                "   as C on Product.ProductID = C.ProductID) " +
                "   where C.ProductID = product.ProductID "
                    ;

            for (int i = 0; i <= categoryID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categoryID[i] ) );
            }

            for (int i = 0; i <= departmentID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( departmentID[i] ) );
            }

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID " +
                    "ORDER BY CreateDate DESC;";
            DataTable table = DataAccess.ExecuteSelect(
                extQuery, DataAccess.ConvertArrayListToDbParameters( parameterList ) );
            return table;

        }
        public DataTable GetFacetResultByCategoryID(
            Culture culture,
            string[] categoryID,
            string[] departmentID,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex
            )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            string productTable = "Product.";
            if (specValueList.Count > 0)
                productTable = "PS.";

            string manufacturerQuery = String.Empty;
            if (!String.IsNullOrEmpty( manufacturerID ))
            {
                manufacturerQuery = "   and c.Manufacturer = " + manufacturerID;
            }
            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByTextGetByCategoryDepartment( sortBy ) + ") as rownum," +
                " C.ProductID, Name, ShortDescription,   " +
                " LongDescription, UrlName, C.Sku,        " +
                " C.Model, C.ImageSecondary, SumStock, C.Weight, C.FixedShippingCost, C.FreeShippingCost, " +
                " C.ShippingCost, C.Manufacturer,        " +
                " C.Keywords,C.Price,        " +
                " C.IsTodaySpecial, C.IsDownloadable, C.DownloadPath, Product.IsEnabled,        " +
                " C.RelatedProducts, C.MerchantRating,     Product.DiscountGroupID As ProductDiscountGroupID,        " +
                " C.CategoryDiscountGroupID,     C.IsGiftCertificate, C.IsFixedPrice, C.ManufacturerPartNumber, C.Upc,            " +
                " C.TaxClassID,     Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, C.UseInventory,        " +
                " C.ProductDetailsLayoutPath, C.IsAffiliate, C.IsCustomPrice, C.IsCallForPrice, Product.Brand, C.ReturnTime, C.IsProductKit,        " +
                " C.IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles,       " +
                " RecurringNumberOfTrialCycles, RecurringTrialAmount       " +
                " FROM    " +
                " (Product INNER JOIN   " +

                " ( SELECT DISTINCT " + productTable + "ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    FROM " +
                "    ((((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " A ON Product.ProductID = A.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON A." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductDepartment PD ON Product.ProductID = PD.ProductID ) " +
                "    LEFT JOIN Department DD ON PD.DepartmentID = DD.DepartmentID )  " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                " WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +

                CreateSpecificationCondition( specValueList, "" ) +

               CreateCategoryDepartmentQuery( categoryID, departmentID, "A." ) +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ConditionPrice( minPrice, maxPrice, "PP." ) +
                "       )  " +
                "   as C on Product.ProductID = C.ProductID) " +
                "   where C.ProductID = product.ProductID " +
                manufacturerQuery;

            for (int i = 0; i <= specValueList.Count - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( specValueList[i].Value ) );
                parameterList.Add( DataAccess.CreateParameterString( specValueList[i].SpecificationItemID ) );
            }

            for (int i = 0; i <= categoryID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categoryID[i] ) );
            }

            for (int i = 0; i <= departmentID.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( departmentID[i] ) );
            }

            if (!String.IsNullOrEmpty( minPrice ))
            {
                parameterList.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( minPrice ) ) );
            }

            if (!String.IsNullOrEmpty( maxPrice ))
            {
                parameterList.Add( DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( maxPrice ) ) );
            }

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            DataTable table = DataAccess.ExecuteSelect(
                extQuery, DataAccess.ConvertArrayListToDbParameters( parameterList ) );
            return table;
        }

        public DataTable GetByCategoryDepartmentID(
            Culture culture,
            string[] categorydepartmentIDs,
            string sortBy,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            string storeID,
            int startIndex,
            int endIndex
        )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByTextGetByCategoryDepartment( sortBy ) + ") as rownum," +
                " C.ProductID, Name, ShortDescription,   " +
                " LongDescription, UrlName, C.Sku,        " +
                " C.Model, C.ImageSecondary, SumStock, C.Weight, C.FixedShippingCost, C.FreeShippingCost,       " +
                " C.ShippingCost, C.Manufacturer,        " +
                " C.Keywords,C.Price,        " +
                " C.IsTodaySpecial, C.IsDownloadable, C.DownloadPath, Product.IsEnabled,        " +
                " C.RelatedProducts, C.MerchantRating,     Product.DiscountGroupID As ProductDiscountGroupID,        " +
                " C." + _tableName + "DiscountGroupID,     C.IsGiftCertificate, C.IsFixedPrice, C.ManufacturerPartNumber, C.Upc,            " +
                " C.TaxClassID,     Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, C.UseInventory,        " +
                " C.ProductDetailsLayoutPath, C.IsAffiliate, C.IsCustomPrice, C.IsCallForPrice, Product.Brand, C.ReturnTime, C.IsProductKit,        " +
                " C.IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles,       " +
                " RecurringNumberOfTrialCycles, RecurringTrialAmount        " +
                "    FROM " +
                " (Product INNER JOIN   " +

                " ( SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    FROM " +
                "    ((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock  " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " C ON Product.ProductID = C.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON C." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
               CreateCategoryDepartmentQuery( categorydepartmentIDs, "C." ) +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) )
                    + " ";
            if (!isIncludeProductKit)
                query += " AND Product.IsProductKit = 0 ";

            query += "       )  " +
                "   as C on Product.ProductID = C.ProductID) " +
                "   where C.ProductID = product.ProductID ";

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            for (int i = 0; i <= categorydepartmentIDs.Length - 1; i++)
            {
                parameterList.Add( DataAccess.CreateParameterString( categorydepartmentIDs[i] ) );
            }
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            DataTable table = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return table;
        }

        public DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            string query = "SELECT DISTINCT ROW_NUMBER() over ( " + CreateSortByTextGetByCategoryDepartment( sortBy ) + ") as rownum," +
                "Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, FreeShippingCost, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    C.SortOrder " +
                "    FROM " +
                "    ((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " C ON Product.ProductID = C.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON C." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
               "WHERE C." + _tableName + "ID = @ID " +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) )
                    + " ";

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            DataTable table = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( categorydepartmentID ),
                DataAccess.CreateParameterBool( true ) );

            return table;
        }

        public DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            string storeID,
            int startIndex,
            int endIndex
        )
        {
            string query = "SELECT DISTINCT ROW_NUMBER() over ( " + CreateSortByTextGetByCategoryDepartment( sortBy ) + ") as rownum," +
                "Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, FreeShippingCost, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                 "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsAffiliate, IsCustomPrice, IsCallForPrice, Brand, ReturnTime, IsProductKit, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    C.SortOrder " +
                "    FROM " +
                "    ((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock  " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " C ON Product.ProductID = C.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON C." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
               "WHERE C." + _tableName + "ID = @ID " +
                    FlagFilterHelper.CreateFlagFilterWithAnd(
                "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) )
                    + " ";
            if (!isIncludeProductKit)
                query += " AND Product.IsProductKit = 0 ";

            string extQuery = "with DataByCategoryDepartmentID as (" + query + ") " +
                    " select * from DataByCategoryDepartmentID where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect(
                extQuery,
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( categorydepartmentID ),
                DataAccess.CreateParameterBool( true ) );

            return table;
        }

        public DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            bool isAffiliate,
            string storeID
    )
        {
            string query = GenerateQueryStringGetByCategoryDepartmentID( isEnabled );
            query = query + " AND Product.IsAffiliate = @IsAffiliate ";
            query = query + " " + CreateSortByTextForProduct( sortBy ) + "; ";

            DataTable table = DataAccess.ExecuteSelect(
                query,
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( categorydepartmentID ),
                DataAccess.CreateParameterBool( true ),
                DataAccess.CreateParameterBool( isAffiliate ) );

            return table;
        }

        public DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            )
        {

            string query = "SELECT DISTINCT Product.ProductID, Name, ShortDescription, LongDescription, UrlName, Sku, " +
                "    Model, ImageSecondary, SumStock, Weight, FixedShippingCost, FreeShippingCost, " +
                "    ShippingCost, Manufacturer, Keywords,PP.Price, " +
                "    IsTodaySpecial, IsDownloadable, DownloadPath, Product.IsEnabled, " +
                "    RelatedProducts, MerchantRating, " +
                "    Product.DiscountGroupID As ProductDiscountGroupID, " +
                "    D.DiscountGroupID As " + _tableName + "DiscountGroupID, " +
                "    IsGiftCertificate, IsFixedPrice, ManufacturerPartNumber, Upc, TaxClassID, " +
                "    Product.Other1, Product.Other2, Product.Other3, Product.Other4, Product.Other5, UseInventory, " +
                "    ProductDetailsLayoutPath, IsCustomPrice, IsCallForPrice, Brand, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                //"    Product" + _tableName + ".SortOrder " + 
                "    C.SortOrder " +
                "    FROM " +
                "    ((((( (Product INNER JOIN (SELECT ProductID, SUM( Stock ) AS SumStock " +
                "    FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) " +
                "    LEFT JOIN " +
                "        (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Product.ProductID = L.ProductID) " +
                "    INNER JOIN (SELECT * FROM ProductPrice WHERE StoreID = @StoreID1 ) as PP ON Product.ProductID = PP.ProductID) " +
                "    LEFT JOIN Product" + _tableName + " C ON Product.ProductID = C.ProductID ) " +
                "    LEFT JOIN " + _tableName + " D ON C." + _tableName + "ID = D." + _tableName + "ID ) " +
                "    LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID ) " +
                "    LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
                "WHERE C." + _tableName + "ID = @ID " +
                FlagFilterHelper.CreateFlagFilterWithAnd(
                    "Product.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByTextForProduct( sortBy ) + "; ";

            DataTable table = DataAccess.ExecuteSelect(
                query,
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( categorydepartmentID ),
                DataAccess.CreateParameterBool( true ) );

            return table;
        }

        public DataTable GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootID,
            string enabledListCondition,
            bool isIncludeProductKit
            )
        {
            ArrayList parameterList = new ArrayList();
            String sqlString = "SELECT TOP " + numberOfResult + " " +
                "Product.ProductID, " +
                " L.Name, L.ShortDescription, L.LongDescription, L.UrlName, " +
                "Product.ImageSecondary, SumStock, " +
                " IsFixedPrice, UseInventory, IsCustomPrice, IsCallForPrice, Brand, " +
                "Product.DiscountGroupID As ProductDiscountGroupID , " +
                "IsRecurring, RecurringInterval, RecurringIntervalUnit, " +
                "RecurringNumberOfCycles, RecurringNumberOfTrialCycles, RecurringTrialAmount, " +
                "M.MetaKeyword AS MetaKeyword, M.MetaDescription AS MetaDescription " +
                "FROM " +
                "(((( ( Product LEFT JOIN ProductRecurring ON Product.ProductID = ProductRecurring.ProductID ) " +
                "LEFT JOIN (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS L ON " +
                "Product.ProductID = L.ProductID) INNER JOIN (SELECT ProductID, SUM( Stock ) " +
                "AS SumStock FROM ProductStock GROUP BY ProductID ) " +
                "AS S ON Product.ProductID = S.ProductID) " +
                " INNER JOIN " +
                "(SELECT SUM (Quantity) AS ProductIDQty, ProductID FROM OrderItem Group By ProductID) " +
                "AS OI ON Product.ProductID = OI.ProductID ) " +
                "INNER JOIN (SELECT DISTINCT ProductID FROM Product" + _tableName + ", " + _tableName + " " +
                "WHERE Product" + _tableName + "." + _tableName + "ID = " + _tableName + "." + _tableName + "ID and RootID = @RootID " +
                enabledListCondition +
                " )AS PC ON Product.ProductID = PC.ProductID ) " +
                "LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID2) M ON Product.ProductID = M.ProductID " +
                "WHERE ";
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( rootID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            if (useStockInventoryControl)
            {
                sqlString += "((SumStock > @outOfStockValue AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion) AND";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }

            sqlString += " Product.IsEnabled = " + DbImplementation.Sql.CreateBoolText( true ) +
            " AND IsParentVisible = @isParentVisible";
            if (!isIncludeProductKit)
                sqlString += " AND Product.IsProductKit = 0 ";
            sqlString += " ORDER BY ProductIDQty DESC ";
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            DataTable tableBestSelling = DataAccess.ExecuteSelect(
                sqlString, DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            return tableBestSelling;
        }

        public DataTable GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootID,
            string enabledListCondition,
            bool isIncludeProductKit
        )
        {
            ArrayList parameterList = new ArrayList();
            string strSQL = "SELECT DISTINCT TOP " + numberOfRandomProducts + " RandID = NEWID(), Product.ProductID, Name, " +
                "ShortDescription, LongDescription, UrlName, Sku, Model, " +
                "ImageSecondary, SumStock, Weight, " +
                "FixedShippingCost, FreeShippingCost, ShippingCost, Manufacturer, Keywords, " +
                "IsTodaySpecial, IsDownloadable, DownloadPath, " +
                "Product.IsEnabled, RelatedProducts, MerchantRating, UseInventory, " +
                "Product.DiscountGroupID As ProductDiscountGroupID, " +
                "IsFixedPrice, ManufacturerPartNumber, Upc, IsCustomPrice, IsCallForPrice, Brand, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                "FROM (((( Product INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) Left JOIN " +
                "    (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS C " +
                "ON Product.ProductID = C.ProductID) " +
                "INNER JOIN (SELECT  Distinct productID FROM Product" + _tableName + "," + _tableName + " " +
                "WHERE Product" + _tableName + "." + _tableName + "ID = " + _tableName + "." + _tableName + "ID and RootID = @RootID " +
                enabledListCondition +
                " ) AS PC ON Product.ProductID = PC.ProductID ) " +
                "LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                "LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
                "WHERE ";
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( rootID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            if (useStockInventoryControl)
            {
                strSQL += "( (SumStock > @SumStock AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion ) AND";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }

            strSQL += " Product.IsEnabled = @ProductIsEnabled AND IsParentVisible = @IsParentVisible  ";
            if (!isIncludeProductKit)
                strSQL += " AND Product.IsProductKit = 0 ";
            strSQL += " ORDER BY NEWID();";

            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            DataTable table = DataAccess.ExecuteSelect( strSQL, DataAccess.ConvertArrayListToDbParameters( parameterList ) );
            return table;
        }

        public DataTable GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootID,
            string enabledListCondition,
            bool isIncludeProductKit
        )
        {
            ArrayList parameterList = new ArrayList();
            string strSQL = "SELECT DISTINCT TOP " + numberOfRandomProducts + " RandID = NEWID(), Product.ProductID, Name, " +
                "ShortDescription, LongDescription, UrlName, Sku, Model, " +
                "ImageSecondary, SumStock, Weight, " +
                "FixedShippingCost, FreeShippingCost, ShippingCost, Manufacturer, Keywords, " +
                "IsTodaySpecial, IsDownloadable, DownloadPath, " +
                "Product.IsEnabled, RelatedProducts, MerchantRating, UseInventory, " +
                "Product.DiscountGroupID As ProductDiscountGroupID, " +
                "IsFixedPrice, ManufacturerPartNumber, Upc, IsCustomPrice, IsCallForPrice, Brand, " +
                // Edit for Recurring.
                "    IsRecurring, RecurringInterval, RecurringIntervalUnit, RecurringNumberOfCycles," +
                "    RecurringNumberOfTrialCycles, RecurringTrialAmount " +
                "FROM (((( Product INNER JOIN ( SELECT ProductID, SUM( Stock ) AS SumStock FROM ProductStock GROUP BY ProductID ) AS S ON Product.ProductID = S.ProductID ) Left JOIN " +
                "    (SELECT * FROM ProductLocale WHERE CultureID = @CultureID) AS C " +
                "ON Product.ProductID = C.ProductID) " +
                "INNER JOIN (SELECT  Distinct productID FROM Product" + _tableName + "," + _tableName + " " +
                "WHERE Product" + _tableName + "." + _tableName + "ID = " + _tableName + "." + _tableName + "ID and RootID = @RootID " +
                enabledListCondition +
                " ) AS PC ON Product.ProductID = PC.ProductID ) " +
                "LEFT JOIN ProductRecurring E ON E.ProductID = Product.ProductID) " +
                "LEFT JOIN (SELECT * FROM ProductMetaInformation " +
                "WHERE CultureID = @Culture2 AND StoreID = @StoreID) M ON Product.ProductID = M.ProductID " +
                "WHERE ";
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( rootID ) );
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );

            if (useStockInventoryControl)
            {
                strSQL += "( (SumStock > @SumStock AND UseInventory = @trueCondition) OR UseInventory = @falseCondotion ) AND";
                parameterList.Add( DataAccess.CreateParameterInt32( outOfStockValue ) );
                parameterList.Add( DataAccess.CreateParameterBool( true ) );
                parameterList.Add( DataAccess.CreateParameterBool( false ) );
            }

            strSQL += " Product.IsEnabled = @ProductIsEnabled AND IsParentVisible = @IsParentVisible  ";
            if (!isIncludeProductKit)
                strSQL += " AND Product.IsProductKit = 0 ";
            strSQL += " ORDER BY NEWID();";

            parameterList.Add( DataAccess.CreateParameterBool( true ) );
            parameterList.Add( DataAccess.CreateParameterBool( true ) );

            DataTable table = DataAccess.ExecuteSelect( strSQL, DataAccess.ConvertArrayListToDbParameters( parameterList ) );
            return table;
        }

        public void UpdateSortOrder( string categoryID, string[] sortedProductIDs )
        {
            for (int i = 0; i < sortedProductIDs.Length; i++)
            {
                DataAccess.ExecuteNonQuery(
                    "UPDATE Product" + _tableName + " SET SortOrder = @SortOrder WHERE ProductID = @ProductID AND  " + _tableName + "ID = @CategoryID;",
                    DataAccess.CreateParameterInt32( ConvertUtilities.ToInt32( i + 1 ) ),
                    DataAccess.CreateParameterInt32( ConvertUtilities.ToInt32( sortedProductIDs[i] ) ),
                    DataAccess.CreateParameterInt32( ConvertUtilities.ToInt32( categoryID ) ) );
            }
        }

        public int GetProductCountByCategoryDepartmentID( string ID )
        {
            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(ProductID) from Product" + _tableName +
                " WHERE " + _tableName + "ID = @ID;",
                DataAccess.CreateParameterString( ID ) );
            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public int GetCountAllWithoutRootCategoryDepartment()
        {
            string amount = DataAccess.ExecuteScalar(
                "SELECT COUNT(" + _tableName + "ID) from " + _tableName +
                " WHERE RootID <> 0 OR Parent" + _tableName + "ID <> 0;" );
            int result;
            if (int.TryParse( amount, out result ))
                return result;
            else
                return 0;
        }

        public Dictionary<string, string> GetMultipleCategoryDepartmentProduct()
        {
            string sql = "SELECT Product" + _tableName + ".ProductID,Product" + _tableName + "." + _tableName + "ID from Product" + _tableName + ","
                + "(SELECT ProductID,Count(" + _tableName + "ID) as " + _tableName + "Count from Product" + _tableName
                + " Group by ProductID Having Count(" + _tableName + "ID) > 1) as ProductDup,Product "
                + "Where Product" + _tableName + ".ProductID = ProductDup.ProductID "
                + "AND Product" + _tableName + ".ProductID = Product.ProductID AND Product.IsEnabled = 1";

            DataTable table = DataAccess.ExecuteSelect( sql );
            Dictionary<string, string> productDup = new Dictionary<string, string>();

            foreach (DataRow row in table.Rows)
            {
                if (!productDup.ContainsKey( row[_tableName + "ID"].ToString() ))
                    productDup.Add( row[_tableName + "ID"].ToString(), row["ProductID"].ToString() );
                else
                    productDup[row[_tableName + "ID"].ToString()] += "," + row["ProductID"].ToString();
            }
            return productDup;
        }

        #endregion
    }
}
