using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using System.Data;
using Vevo.Shared.Utilities;
using System.Collections;
using Vevo.Domain.Stores;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class StoreRepositoryNoCache : IStoreRepository
    {

        #region Private
        private const string AllStoreFields = "StoreID, StoreName, UrlName, IsEnabled ";

        private Store Create( Store store )
        {
            string newID = DataAccess.ExecuteScalar(
                    "INSERT INTO Store ( StoreName, UrlName, IsEnabled ) " +
                    "VALUES ( @StoreName, @UrlName, @IsEnabled ); " +
                    "SELECT @@Identity; ",
                    DataAccess.CreateParameterString( store.StoreName ),
                    DataAccess.CreateParameterString( store.UrlName ),
                    DataAccess.CreateParameterBool( store.IsEnabled )
                     );
            store.StoreID = newID;
            return store;
        }

        private Store Update( Store store )
        {
            DataAccess.ExecuteNonQuery( "UPDATE Store " +
               "SET StoreName = @StoreName, UrlName=@UrlName, IsEnabled=@IsEnabled " +
               "WHERE StoreID = @StoreID; ",
              DataAccess.CreateParameterString( store.StoreName ),
              DataAccess.CreateParameterString( store.UrlName ),
              DataAccess.CreateParameterBool( store.IsEnabled ),
              DataAccess.CreateParameterString( store.StoreID ) );

            return store;
        }

        private Store CreateOneFromRow( DataRow row )
        {
            Store store = new Store();

            store.StoreID = row["StoreID"].ToString();
            store.StoreName = row["StoreName"].ToString();
            store.UrlName = row["UrlName"].ToString();
            store.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );

            return store;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private IList<Store> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Store> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Store> list = new List<Store>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion

        #region Constructors

        public StoreRepositoryNoCache()
        {
        }

        #endregion

        #region IStoreRepository Members

        public Store Save( Store store )
        {
            Store result;

            if (store.StoreID == "0")
                result = Create( store );
            else
                result = Update( store );

            return result;
        }

        public void Delete( string storeID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Store " +
                "WHERE StoreID = @StoreID ;",
                DataAccess.CreateParameterString( storeID ) );
        }

        public Store GetOne( string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllStoreFields +
                "FROM Store " +
                "WHERE StoreID = @StoreID; ",
                DataAccess.CreateParameterString( storeID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Store.Null;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT StoreID, StoreName, UrlName, IsEnabled " +
                 "FROM Store " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }


        public IList<Store> SearchStore(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllStoreFields +
                "FROM Store " + commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["StoreID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table, startIndex, endIndex );
        }

        public string GetStoreIDByUrlName( string urlName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM Store " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( urlName ) );
            
            if (table.Rows.Count == 0)
                return "0";
            else
                return table.Rows[0]["StoreID"].ToString();

        }

        public string GetStoreIDByStoreName( string storeName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM Store " +
                "WHERE StoreName = @StoreName; ",
                DataAccess.CreateParameterString( storeName ) );

            if (table.Rows.Count == 0)
                return "0";
            else
                return table.Rows[0]["StoreID"].ToString();
        }

        public IList<Store> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllStoreFields +
                "FROM Store " +
                CreateSortByText( sortBy ) + "; " );
            return ConvertTableToList( table );
        }

        #endregion
    }
}
