using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using GCheckout.MerchantCalculation;
using GCheckout.Util;
using Vevo;
using Vevo.Domain.Discounts;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Payments.Google;
using Vevo.Domain.Payments.SessionState;
using Vevo.Domain.Tax;
using Vevo.Domain.Users;
using Vevo.Domain.Shipping;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain.Orders;

namespace Vevo.Domain.Payments.Google
{
    /// <summary>
    /// Summary description for CallBackRulesDetails
    /// </summary>
    public class VevoCallBackRules : CallbackRules
    {
        #region Private
        private string _shippingName;

        private string ShippingName
        {
            get
            {
                return _shippingName;
            }
            set
            {
                _shippingName = value;
            }
        }

        private Session GetSession( GCheckout.MerchantCalculation.Order thisOrder )
        {
            string ticket = FindElement(
                thisOrder.MerchantPrivateDataNodes,
                PaymentOption.GoogleCheckoutSessionTicketName );

            return SessionPool.GetSession( ticket );
        }

        private CheckoutDetails GetCheckoutDetails( GCheckout.MerchantCalculation.Order thisOrder )
        {
            Session session = GetSession( thisOrder );
            return (CheckoutDetails) session[GoogleCheckoutPaymentMethod.CheckoutDetailsSessionName];
        }

        private Cart GetCart( GCheckout.MerchantCalculation.Order thisOrder )
        {
            Session session = GetSession( thisOrder );
            return (Cart) session[GoogleCheckoutPaymentMethod.CartSessionName];
        }

        public string FindElement( XmlNode[] nodes, string elementName )
        {
            foreach (XmlNode node in nodes)
            {
                if (String.Compare( node.Name, elementName, true ) == 0)
                {
                    return node.InnerText;
                }
            }
            return String.Empty;
        }

        private bool CouponExisted(
            GCheckout.MerchantCalculation.Order thisOrder,
            string merchantCode )
        {
            return !String.IsNullOrEmpty( GetCheckoutDetails( thisOrder ).Coupon.CouponID ) &&
                GetCheckoutDetails( thisOrder ).Coupon.CouponID != merchantCode;
        }

        private bool GiftCertificateExisted(
            GCheckout.MerchantCalculation.Order thisOrder,
            string merchantCode )
        {
            string giftCertificateCode = GetCheckoutDetails( thisOrder ).GiftCertificate.GiftCertificateCode;

            return !String.IsNullOrEmpty( giftCertificateCode )
                && giftCertificateCode != merchantCode;
        }

        private MerchantCodeResult CreateValidResult( MerchantCodeType type, decimal amount )
        {
            MerchantCodeResult result = new MerchantCodeResult();
            result.Type = type;
            result.Valid = true;
            result.Amount = amount;

            return result;
        }

        private MerchantCodeResult CreateInvalidResult( MerchantCodeType type, string errorMessage )
        {
            MerchantCodeResult result = new MerchantCodeResult();
            result.Type = type;
            result.Valid = false;
            result.Amount = 0;
            result.Message = errorMessage;

            return result;
        }

        private MerchantCodeResult CreateInvalidCouponResult( string errorMessage )
        {
            return CreateInvalidResult( MerchantCodeType.Coupon, errorMessage );
        }

        private MerchantCodeResult CreateInvalidGiftCertificateResult( string errorMessage )
        {
            return CreateInvalidResult( MerchantCodeType.GiftCertificate, errorMessage );
        }

        private bool IsCouponCode( string merchantCode )
        {
            Coupon coupon = DataAccessContext.CouponRepository.GetOne( merchantCode );
            return coupon.ExpirationStatus != Coupon.ExpirationStatusEnum.UnknownType;
        }

        private bool IsGiftCertificate( string merchantCode )
        {
            GiftCertificate giftCertificate = DataAccessContext.GiftCertificateRepository.GetOne( merchantCode );
            return !giftCertificate.IsNull;
        }

        private bool IsCouponExpired( string merchantCode )
        {
            Coupon coupon = DataAccessContext.CouponRepository.GetOne( merchantCode );
            return coupon.ExpirationStatus != Coupon.ExpirationStatusEnum.NotExpired;
        }

        private ShippingAddress ConvertToAddress( AnonymousAddress address )
        {
            return Address.GetFactory()
                .WithCity( address.City )
                .WithState( address.Region )
                .WithZip( address.PostalCode )
                .WithCountry( address.CountryCode )
                .CreateShippingAddress( false );
        }

        private OrderAmount GetOrderAmount( GCheckout.MerchantCalculation.Order thisOrder )
        {
            CheckoutDetails checkoutDetails = GetCheckoutDetails( thisOrder );
            Cart cart = GetCart( thisOrder );

            OrderCalculator orderCalculator = new OrderCalculator();
            return orderCalculator.Calculate(
                checkoutDetails,
                cart.SeparateCartItemGroups(),
                Customer.Null ).Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                checkoutDetails,
                cart.SeparateCartItemGroups(),
                Customer.Null));
        }

        private decimal GetCouponDiscount( GCheckout.MerchantCalculation.Order thisOrder )
        {
            CheckoutDetails checkoutDetails = GetCheckoutDetails( thisOrder );
            Cart cart = GetCart( thisOrder );
            IList<decimal> discountLines;
            decimal discount = 0m;
            
            if (checkoutDetails.Coupon.DiscountType == Coupon.DiscountTypeEnum.FreeShipping)
            {
                discount = GetShippingResult( ShippingName, thisOrder, null ).ShippingRate;
            }
            else
            { 
                discount = checkoutDetails.Coupon.GetDiscount( cart.GetAllCartItems(), Customer.Null, out discountLines );
            }

            return discount;
        }

        private MerchantCodeResult CreateCouponResult(
            GCheckout.MerchantCalculation.Order thisOrder,
            string merchantCode )
        {
            if (CouponExisted( thisOrder, merchantCode ))
            {
                return CreateInvalidCouponResult( "Only one coupon can be applied for this order." );
            }
            else if (IsCouponExpired( merchantCode ))
            {
                return CreateInvalidCouponResult( "This coupon has expired." );
            }
            else
            {
                GetCheckoutDetails( thisOrder ).Coupon = DataAccessContext.CouponRepository.GetOne( merchantCode );

                return CreateValidResult(
                    MerchantCodeType.Coupon,
                    GetCouponDiscount( thisOrder ) );
            }
        }

        private MerchantCodeResult CreateGiftCertificateResult(
            GCheckout.MerchantCalculation.Order thisOrder,
            string merchantCode
            )
        {
            string message;

            GiftCertificate giftCertificate = DataAccessContext.GiftCertificateRepository.GetOne( merchantCode );

            if (GiftCertificateExisted( thisOrder, merchantCode ))
            {
                return CreateInvalidGiftCertificateResult(
                    "Only one gift certificate can be applied for this order." );
            }
            else if (!giftCertificate.Verify( out message ))
            {
                return CreateInvalidGiftCertificateResult( message );
            }
            else
            {
                GetCheckoutDetails( thisOrder ).GiftCertificate = DataAccessContext.GiftCertificateRepository.GetOne( merchantCode );

                return CreateValidResult(
                    MerchantCodeType.GiftCertificate,
                    GetOrderAmount( thisOrder ).GiftCertificate );
            }
        }

        #endregion


        public override MerchantCodeResult GetMerchantCodeResult(
            GCheckout.MerchantCalculation.Order thisOrder,
            AnonymousAddress address,
            string merchantCode )
        {
            Log.Debug( "use GetMerchantCodeResult" );

            MerchantCodeResult result;

            if (IsCouponCode( merchantCode ))
            {
                result = CreateCouponResult( thisOrder, merchantCode );
            }
            else if (IsGiftCertificate( merchantCode ))
            {
                result = CreateGiftCertificateResult( thisOrder, merchantCode );
            }
            else
            {
                result = CreateInvalidResult(
                    MerchantCodeType.Undefined,
                    "This code is not a valid coupon or gift certificate code" );
            }

            return result;
        }

        public override ShippingResult GetShippingResult(
            string ShipMethodName,
            GCheckout.MerchantCalculation.Order thisOrder,
            AnonymousAddress address )
        {
            Log.Debug( "use GetShippingResult" );
            ShippingResult result = new ShippingResult();
            result.Shippable = true;
            ShippingName = ShipMethodName;

            string shippingID = DataAccessContext.ShippingOptionRepository.GetIDFromName( ShipMethodName );

            CartItemGroup cartItemGroup = GetCart( thisOrder ).GetAllCartItems();
            ShippingOption shippingOption = DataAccessContext.ShippingOptionRepository.GetOne( Culture.Null, shippingID );
            ShippingMethod shippingMethod = shippingOption.CreateNonRealTimeShippingMethod();
            result.ShippingRate = shippingMethod.GetShippingCost( cartItemGroup, WholesaleStatus.Null, 0m )
                + CartItemPromotion.GetShippingCostFromPromotion( shippingMethod,
                    cartItemGroup,
                    WholesaleStatus.Null,
                    0.0m );

            return result;
        }

        public override decimal GetTaxResult(
            GCheckout.MerchantCalculation.Order thisOrder,
            AnonymousAddress address,
            decimal shippingRate )
        {
            Log.Debug( "use GetTaxResult" );

            string subtotal = FindElement( thisOrder.MerchantPrivateDataNodes, "subtotal" );
            string lineItemTaxes = FindElement( thisOrder.MerchantPrivateDataNodes, "totalLineItemTaxes" );

            CheckoutDetails checkoutDetails = GetCheckoutDetails( thisOrder );
            checkoutDetails.ShippingAddress = ConvertToAddress( address );
            Cart cart = GetCart( thisOrder );

            OrderCalculator orderCalculator = new OrderCalculator();
            OrderAmount orderAmount = orderCalculator.Calculate(
                checkoutDetails,
                cart.SeparateCartItemGroups(),
                Customer.Null ).Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                checkoutDetails,
                cart.SeparateCartItemGroups(),
                Customer.Null ) );;


            decimal amount = orderCalculator.CalculateGoogleCheckoutTax(
                                checkoutDetails, cart.SeparateCartItemGroups(), Customer.Null, shippingRate );

            Log.Debug( "GetTaxResult - Tax amount = " + amount );

            return amount;
        }

    }
}
