using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;

using Vevo.Domain.Exceptions;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;

namespace Vevo.Domain.Configurations
{
    public class ConfigurationCollection : SortedList<string, Configuration>
    {
        #region Private

        private IStoreRetriever _storeRetriever = new StoreRetriever();

        private string GetValueNoThrowPrivate( string name, out bool found )
        {
            return GetValueNoThrowPrivate( Culture.Null.CultureID, name, out found );
        }

        private string GetValueNoThrowPrivate( string name, Store store, out bool found )
        {
            return GetValueNoThrowPrivate( Culture.Null.CultureID, name, store, out found );
        }

        private string GetValueNoThrowPrivate( string cultureID, string name, out bool found )
        {
            if (!Store.CanUseMultiStoreConfig( name ))
                return GetValueNoThrowPrivate( cultureID, name, Store.Null.StoreID, out found );
            else
                return GetValueNoThrowPrivate( cultureID, name, _storeRetriever.GetCurrentStoreID(), out found );

        }

        private string GetValueNoThrowPrivate( string cultureID, string name, Store store, out bool found )
        {
            return GetValueNoThrowPrivate( cultureID, name, store.StoreID, out found );
        }

        private string GetValueNoThrowPrivate( string cultureID, string name, string storeID, out bool found )
        {

            if (this.ContainsKey( name ))
            {
                found = true;
                Configuration config = this[name];

                ConfigurationValue configValue = config.FindValueByCultureIDAndStoreID( cultureID, storeID );
                if (configValue != null)
                    return configValue.ItemValue;
                else
                    return String.Empty;
            }

            found = false;
            return String.Empty;
        }

        #endregion


        #region Constructors

        public ConfigurationCollection()
            : base( StringComparer.CurrentCultureIgnoreCase )
        {
        }

        public ConfigurationCollection( IStoreRetriever storeRetriever )
            : base( StringComparer.CurrentCultureIgnoreCase )
        {
            _storeRetriever = storeRetriever;
        }

        public ConfigurationCollection( IList<Configuration> configurations )
            : base( StringComparer.CurrentCultureIgnoreCase )
        {
            foreach (Configuration config in configurations)
            {
                if (!ContainsKey( config.Name ))
                    Add( config.Name, config );
            }
        }

        public ConfigurationCollection( IList<Configuration> configurations, IStoreRetriever storeRetriever )
            : base( StringComparer.CurrentCultureIgnoreCase )
        {
            _storeRetriever = storeRetriever;
            foreach (Configuration config in configurations)
            {
                if (!ContainsKey( config.Name ))
                    Add( config.Name, config );
            }
        }
        #endregion


        #region Public Methods

        public Configuration AddNewConfig( string configName )
        {
            Configuration config = new Configuration( configName );
            this.Add( configName, config );
            return config;
        }

        public Configuration AddNewConfig( string configName, string value )
        {
            Configuration config = AddNewConfig( configName );
            config.SetValue( value );
            return config;
        }


        public Configuration AddNewConfig( string configName, string value, Store store )
        {
            Configuration config;
            if (!this.ContainsKey( configName ))
                config = AddNewConfig( configName );
            else
                config = this[configName];
            config.SetValue( value, store );

            return config;
        }

        public Configuration AddNewConfig( string configName, string cultureID, string value, Store store )
        {
            Configuration config;
            if (!this.ContainsKey( configName ))
                config = AddNewConfig( configName );
            else
                config = this[configName];
            config.SetValue( cultureID, value, store.StoreID );

            return config;
        }

        public string GetValue( string configName )
        {
            return GetValue( Culture.Null.CultureID, configName );
        }

        public string GetValue( string configName, Store store )
        {
            return GetValue( Culture.Null.CultureID, configName, store );
        }

        public string GetValue( string cultureID, string configName )
        {
            if (!Store.CanUseMultiStoreConfig( configName ))
                return GetValue( cultureID, configName, Store.Null.StoreID );
            else
                return GetValue( cultureID, configName, _storeRetriever.GetCurrentStoreID() );


            //  return GetValue( cultureID, configName, Store.Null );
        }

        public string GetValue( string cultureID, string configName, Store store )
        {
            return GetValue( cultureID, configName, store.StoreID );
        }

        public string GetValue( string cultureID, string configName, string storeID )
        {
            bool found;
            string result = GetValueNoThrowPrivate( cultureID, configName, storeID, out found );

            if (found)
                return result;
            else
                throw new ConfigurationException( "Unknown configuration name: " + configName );
        }

        public bool GetBoolValue( string configName )
        {
            string valueString = GetValue( configName );

            if (!string.IsNullOrEmpty( valueString ))
                return Boolean.Parse( valueString );
            else
                return false;
        }

        public bool GetBoolValue( string configName, Store store )
        {
            string valueString = GetValue( configName, store );

            if (!string.IsNullOrEmpty( valueString ))
                return Boolean.Parse( valueString );
            else
                return false;
        }

        public bool GetBoolValueNoThrow( string configName )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, out found );

            bool valueBool;
            if (!string.IsNullOrEmpty( valueString ) &&
                Boolean.TryParse( valueString, out valueBool ))
                return valueBool;
            else
                return false;
        }

        public bool GetBoolValueNoThrow( string configName, Store store )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, store, out found );

            bool valueBool;
            if (!string.IsNullOrEmpty( valueString ) &&
                Boolean.TryParse( valueString, out valueBool ))
                return valueBool;
            else
                return false;
        }

        public double GetDoubleValue( string configName )
        {
            string valueString = GetValue( configName );

            if (!string.IsNullOrEmpty( valueString ))
                return ConvertUtilities.EnglishStringToDouble( valueString );
            else
                return 0;
        }

        public double GetDoubleValueNoThrow( string configName )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, out found );

            double result;
            if (!string.IsNullOrEmpty( valueString ) &&
                Double.TryParse( valueString, out result ))
                return ConvertUtilities.EnglishStringToDouble( valueString );
            else
                return 0;
        }

        public int GetIntValue( string configName )
        {
            string valueString = GetValue( configName );

            if (!string.IsNullOrEmpty( valueString ))
                return Int32.Parse( valueString );
            else
                return 0;
        }

        public int GetIntValue( string configName, Store store )
        {
            string valueString = GetValue( configName, store );

            if (!string.IsNullOrEmpty( valueString ))
                return Int32.Parse( valueString );
            else
                return 0;
        }

        public int GetIntValueNoThrow( string configName )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, out found );

            int valueInt;
            if (!string.IsNullOrEmpty( valueString ) &&
                int.TryParse( valueString, out valueInt ))
                return valueInt;
            else
                return 0;
        }

        public decimal GetDecimalValue( string configName )
        {
            string valueString = GetValue( configName );

            if (!string.IsNullOrEmpty( valueString ))
                return ConvertUtilities.EnglishStringToDecimal( valueString );
            else
                return 0;
        }

        public decimal GetDecimalValueNoThrow( string configName )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, out found );

            decimal valueDecimal;
            if (!string.IsNullOrEmpty( valueString ) &&
                decimal.TryParse( valueString, out valueDecimal ))
                return ConvertUtilities.EnglishStringToDecimal( valueString );
            else
                return 0;
        }

        public string[] GetValueList( string configName )
        {
            string valueString = GetValue( configName );

            if (!string.IsNullOrEmpty( valueString ))
                return StringUtilities.SplitString( valueString, ',' );
            else
                return new string[0];
        }

        public string[] GetValueListNoThrow( string configName )
        {
            bool found;
            string valueString = GetValueNoThrowPrivate( configName, out found );

            if (!string.IsNullOrEmpty( valueString ))
                return StringUtilities.SplitString( valueString, ',' );
            else
                return new string[0];
        }

        public string GetValueNoThrow( string configName )
        {
            return GetValueNoThrow( Culture.Null.CultureID, configName );
        }

        public string GetValueNoThrow( string configName, Store store )
        {
            bool found;
            return GetValueNoThrowPrivate( configName, store, out found );
        }

        public string GetValueNoThrow( string cultureID, string configName )
        {
            bool found;
            return GetValueNoThrowPrivate( cultureID, configName, out found );
        }

        public string GetValueNoThrow( string cultureID, string configName, Store store )
        {
            bool found;
            return GetValueNoThrowPrivate( cultureID, configName, store, out found );
        }

        public NameValueCollection GetNameValueCollectionSingleCulture()
        {
            NameValueCollection nameValueCollection = new NameValueCollection();
            foreach (Configuration configuration in Values)
            {
                if (configuration.CanUseSingleCulture())
                    nameValueCollection.Add( configuration.Name, configuration.Values[0].ItemValue );
            }

            return nameValueCollection;
        }

        public NameValueCollection GetNameValueCollectionSingleCultureByStore( Store store )
        {
            NameValueCollection nameValueCollection = new NameValueCollection();
            foreach (Configuration configuration in Values)
            {
                if (configuration.CanUseSingleCulture())
                    foreach (ConfigurationValue configurationValue in configuration.Values)
                    {
                        if (Store.CanUseMultiStoreConfig( configuration.Name ))
                        {
                            if (configurationValue.StoreID == store.StoreID)
                                nameValueCollection.Add( configuration.Name, configurationValue.ItemValue );
                        }
                        else
                        {
                            nameValueCollection.Add( configuration.Name, configuration.Values[0].ItemValue );
                        }
                    }
            }

            return nameValueCollection;
        }

        #endregion

    }
}
