using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Base;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Contents
{
    public class ContentMenuItem : ILocaleLoader, ICloneable
    {
        #region Public Constances

        public enum MenuPositionType
        {
            Root,
            Top,
            Left,
            Right
        }

        public const string RootMenuItemID = "1";
        public const string RootMenuItemName = "[$RootMenuName]";

        #endregion

        #region Private

        private string _contentMenuItemID = "0";
        private string _contentMenuID = "0";
        private string _contentID = "0";
        private string _referringMenuID = "0";
        private int _sortOrder = 0;
        private bool _isEnabled = true;
        private string _other1 = String.Empty;
        private string _other2 = String.Empty;
        private string _other3 = String.Empty;
        private MenuPositionType _menuPosition;
        private string _storeID = "0";

        private Culture _defaultCulture;
        private LocaleCollection<ContentMenuItemLocale> _locales;


        private ContentMenuItem GetReferringContentMenuItem( Culture culture, string id )
        {
            IList<ContentMenuItem> list = DataAccessContext.ContentMenuItemRepository.GetAll( culture, BoolFilter.ShowAll );
            foreach (ContentMenuItem item in list)
            {
                if (item.ReferringMenuID == id)
                    return item;
            }
            return null;
        }

        private IList<ContentMenuItem> GenerateParent(
            Culture culture, string contentMenuItemID, IList<ContentMenuItem> parentList )
        {
            ContentMenuItem item = DataAccessContext.ContentMenuItemRepository.GetOne(
                culture, contentMenuItemID );

            if (item.MenuPosition != MenuPositionType.Root)
            {
                parentList.Insert( 0, item );
                ContentMenuItem nextitem = GetReferringContentMenuItem( culture, item.ContentMenuID.ToString() );
                if (nextitem.MenuPosition != MenuPositionType.Root)
                    GenerateParent( culture, nextitem.ContentMenuItemID, parentList );
            }
            else
            {
                parentList.Insert( 0, item );
            }

            return parentList;
        }

        #endregion

        #region Public Constructors

        public ContentMenuItem( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<ContentMenuItemLocale>( this );
        }
        #endregion

        #region Public Properties

        public string ContentMenuItemID { get { return _contentMenuItemID; } set { _contentMenuItemID = value; } }
        public string ContentMenuID { get { return _contentMenuID; } set { _contentMenuID = value; } }
        public string ReferringMenuID { get { return _referringMenuID; } set { _referringMenuID = value; } }
        public string ContentID { get { return _contentID; } set { _contentID = value; } }
        public int SortOrder { get { return _sortOrder; } set { _sortOrder = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string Other1 { get { return _other1; } set { _other1 = value; } }
        public string Other2 { get { return _other2; } set { _other2 = value; } }
        public string Other3 { get { return _other3; } set { _other3 = value; } }
        public MenuPositionType MenuPosition { get { return _menuPosition; } set { _menuPosition = value; } }
        public string StoreID { get { return _storeID; } set { _storeID = value; } }
        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<ContentMenuItemLocale> Locales { get { return _locales; } }

        public static ContentMenuItem Null
        {
            get { return NullContentMenuItem.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Name; }
            set { Locales.GetNoLoad( DefaultCulture ).Name = value; }
        }

        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }

        public string UrlName
        {
            get { return Locales[DefaultCulture].UrlName; }
            set { Locales.GetNoLoad( DefaultCulture ).UrlName = value; }
        }

        // End Locale items
        //------------------------------------------------------------        
        #endregion

        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ContentMenuItemLocale locale in DataAccessContext.ContentMenuItemRepository.GetLocales( ContentMenuItemID ))
                result.Add( locale );

            return result;
        }

        #endregion

        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            ContentMenuItem newObject = (ContentMenuItem) MemberwiseClone();
            newObject._locales = (LocaleCollection<ContentMenuItemLocale>) Locales.Clone();
            return newObject;
        }

        #endregion

        #region Public Methods

        public bool LinksToContent()
        {
            return ConvertUtilities.ToInt32( ContentID ) != 0;
        }

        public string GetFullContentPath( Culture culture )
        {
            IList<ContentMenuItem> parentList = new List<ContentMenuItem>();

            parentList = GenerateParent( culture, ContentMenuItemID, parentList );
            string name = parentList[0].Name;

            for (int i = 1; i < parentList.Count; i++)
            {
                name += " >> " + parentList[i].Name;
            }
            return name;

        }

        public IList<ContentMenuItem> GetParentMenuItemList( Culture culture )
        {
            IList<ContentMenuItem> parentList = new List<ContentMenuItem>();
            parentList = GenerateParent( culture, ContentMenuItemID, parentList );

            return parentList;
        }

        #endregion
    }
}
