using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Discounts
{
    public class Coupon : ICloneable
    {
        #region Public Enun Type

        public enum DiscountTypeEnum
        {
            Price,
            Percentage,
            BuyXDiscountYPrice,
            BuyXDiscountYPercentage,
            FreeShipping
        }

        public enum ExpirationTypeEnum
        {
            Date,
            Quantity,
            Both
        }

        public enum ExpirationStatusEnum
        {
            NotExpired,
            ExpiredByDate,
            ExpiredByQuantity,
            UnknownType
        }

        public enum ProductFilterEnum
        {
            All,
            ByProductIDs,
            ByCategoryIDs
        }

        public enum CustomerFilterEnum
        {
            All,
            ByUserNames
        }

        public enum ProductCostTypeEnum
        {
            AnyProduct,
            CouponEligibleProducts
        }

        public enum ValidationStatus
        {
            OK,
            InvalidCode,
            Expired,
            BelowMinimumOrderAmount,
            InvalidCustomer,
            InvalidProduct,
            BelowMinimumQuantity
        }

        public enum FreeShippingTypeEnum
        {
            MatchingItemsOnly,
            ShipmentWithMatchingItem
        }

        #endregion


        #region Private

        private delegate bool VerifyCouponCallback( Product product, Customer customer );


        private string _couponID = String.Empty;
        private DiscountTypeEnum _discountType = DiscountTypeEnum.Price;
        private decimal _discountAmount = 0;
        private double _percentage = 0;
        private bool _repeatDiscount = false;
        private int _minimumQuantity = 0;
        private int _promotionQuantity = 0;
        private ExpirationTypeEnum _expirationType = ExpirationTypeEnum.Quantity;
        private DateTime _expirationDate = DateTime.Today;
        private int _expirationQuantity = 0;
        private int _currentQuantity = 0;
        private string _merchantNotes = String.Empty;
        private ProductFilterEnum _productFilter = ProductFilterEnum.All;
        private string _productIDs = String.Empty;
        private string _categoryIDs = String.Empty;
        private CustomerFilterEnum _customerFilter = CustomerFilterEnum.All;
        private string _customerUserName = String.Empty;
        private decimal _minimumSubtotal = 0;
        private ProductCostTypeEnum _productCostType = ProductCostTypeEnum.AnyProduct;
        private FreeShippingTypeEnum _freeShippingType = FreeShippingTypeEnum.MatchingItemsOnly;


        private bool IsCouponExpired()
        {
            return (ExpirationStatus != Coupon.ExpirationStatusEnum.NotExpired);
        }

        private bool VerifyCouponNoExpiration( Product product, Customer customer )
        {
            if (ConvertUtilities.ToInt32( product.DiscountGroupID ) == 0
                && IsProductDiscountable( product )
                && IsCustomerDiscountable( customer ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool VerifyCoupon( Product product, Customer customer )
        {
            return VerifyCouponNoExpiration( product, customer )
               && !IsCouponExpired();
        }

        private int GetRepeatDiscountTimes( int itemQuantity )
        {
            if (RepeatDiscount)
            {
                return itemQuantity / (PromotionQuantity + MinimumQuantity);
            }
            else
            {
                return 1;
            }
        }

        private decimal GetDiscountAmount( decimal itemPrice, decimal subtotal, ICartItem item )
        {
            decimal discount = 0;
            decimal percentage = Convert.ToDecimal( Percentage ) / 100m;

            switch (DiscountType)
            {
                case Coupon.DiscountTypeEnum.Price:
                    discount = DiscountAmount * itemPrice / subtotal;
                    break;
                case Coupon.DiscountTypeEnum.Percentage:
                    discount = itemPrice * percentage;
                    break;
                case Coupon.DiscountTypeEnum.BuyXDiscountYPrice:
                    discount = GetRepeatDiscountTimes( item.Quantity ) * DiscountAmount;
                    break;
                case Coupon.DiscountTypeEnum.BuyXDiscountYPercentage:
                    discount = GetRepeatDiscountTimes( item.Quantity ) * (itemPrice / item.Quantity) * percentage * PromotionQuantity;
                    break;
            }

            if (discount > itemPrice)
                discount = itemPrice;

            return discount;
        }

        private decimal GetSubTotal( ICartItem[] cart, Customer customer, VerifyCouponCallback verifyCouponCallback )
        {
            decimal subtotal = 0;
            foreach (ICartItem item in cart)
            {
                if (item.IsPromotion)
                {
                    subtotal += item.GetSubtotal( customer.GetWholesaleStatus() );
                }
                else if (verifyCouponCallback( item.Product, customer ) && !item.IsRecurring)
                {
                    subtotal += item.GetSubtotal( customer.GetWholesaleStatus() );
                }

            }

            return subtotal;
        }

        private decimal GetSubTotalWithoutPromotion( ICartItem[] cart, Customer customer, VerifyCouponCallback verifyCouponCallback )
        {
            decimal subtotal = 0;
            foreach (ICartItem item in cart)
            {
                if ((!item.IsPromotion) && (verifyCouponCallback( item.Product, customer ) && !item.IsRecurring))
                {
                    subtotal += item.GetSubtotal( customer.GetWholesaleStatus() );
                }
            }

            return subtotal;
        }

        private bool IsDiscount( int itemQty )
        {
            if (DiscountType == Coupon.DiscountTypeEnum.Percentage ||
                DiscountType == Coupon.DiscountTypeEnum.Price)
            {
                return true;
            }

            bool itemIsDiscount = false;

            if (itemQty > MinimumQuantity)
                itemIsDiscount = true;

            return itemIsDiscount;
        }

        private decimal GetDiscountPerOrder(
            ICartItem[] cart,
            Customer customer,
            VerifyCouponCallback verifyCouponCallback,
            out IList<decimal> discountLines )
        {
            decimal subtotal = 0;
            decimal subtotalWithoutPromotion = 0;
            decimal discountTotal = 0;
            discountLines = new List<decimal>();
            subtotal = GetSubTotal( cart, customer, verifyCouponCallback );
            subtotalWithoutPromotion = GetSubTotalWithoutPromotion( cart, customer, verifyCouponCallback );

            foreach (ICartItem item in cart)
            {
                decimal discount = 0;
                if (!item.IsPromotion)
                {
                    if (verifyCouponCallback( item.Product, customer ) && !item.IsRecurring && IsDiscount( item.Quantity ))
                    {
                        discount = GetDiscountAmount(
                            item.GetSubtotal( customer.GetWholesaleStatus() ), subtotalWithoutPromotion, item );
                        discountTotal += discount;
                    }
                }
                else
                {
                    discount = 0;
                    discountTotal += discount;
                }
                discountLines.Add( discount );
            }

            return discountTotal;
        }

        private bool CheckExistInList( string stringList, string stringSearch )
        {
            string[] stringListSplit = stringList.Split( new char[] { ',' } );
            foreach (string item in stringListSplit)
            {
                if (item.ToLower().Trim() == stringSearch.ToLower().Trim())
                    return true;
            }
            return false;
        }

        private bool CheckExistInList( string stringList, string[] stringSearch )
        {
            string[] stringListSplit = stringList.Split( new char[] { ',' } );
            foreach (string item in stringListSplit)
            {
                foreach (string itemSearch in stringSearch)
                {
                    if (item.ToLower().Trim() == itemSearch.ToLower().Trim())
                        return true;
                }
            }
            return false;
        }

        private bool QualifyQuantity( CartItemGroup cartItems )
        {
            if (DiscountType != DiscountTypeEnum.BuyXDiscountYPrice &&
                 DiscountType != Coupon.DiscountTypeEnum.BuyXDiscountYPercentage)
            {
                return true;
            }

            foreach (ICartItem item in cartItems.GetCartItemArray())
            {
                if (IsProductDiscountable( item.Product ))
                {
                    if (item.Quantity >= (MinimumQuantity + PromotionQuantity))
                    {
                        return true;
                    }
                }
            }

            return false;
        }


        private void AddParentsOfCategoryToList(
            string categoryID,
            ArrayList categoryIDs )
        {
            Category category = DataAccessContext.CategoryRepository.GetOne( Culture.Null, categoryID );

            if (category.ParentCategoryID != "0")
            {
                categoryIDs.Add( category.ParentCategoryID );

                AddParentsOfCategoryToList(
                    categoryIDs[categoryIDs.Count - 1].ToString(), categoryIDs );
            }
        }

        private string[] ConvertToArray( IList<string> list )
        {
            string[] array = new string[list.Count];
            list.CopyTo( array, 0 );

            return array;
        }

        private string[] AddParentCategoriesToList( IList<string> originalCategoryIDs )
        {
            ArrayList parentCategoryIDs = new ArrayList();

            for (int i = 0; i < originalCategoryIDs.Count; i++)
            {
                AddParentsOfCategoryToList( originalCategoryIDs[i], parentCategoryIDs );
            }

            ArrayList merged = new ArrayList( ConvertToArray( originalCategoryIDs ) );
            merged.AddRange( parentCategoryIDs );

            string[] result = new string[merged.Count];
            merged.CopyTo( result );

            return result;
        }

        private ExpirationStatusEnum GetExpirationStatusByDate()
        {
            if (DateTime.Today > ExpirationDate)
                return ExpirationStatusEnum.ExpiredByDate;
            else
                return ExpirationStatusEnum.NotExpired;
        }

        private ExpirationStatusEnum GetExpirationStatusByQuantity()
        {
            if (CurrentQuantity >= ExpirationQuantity)
                return ExpirationStatusEnum.ExpiredByQuantity;
            else
                return ExpirationStatusEnum.NotExpired;
        }

        #endregion


        #region Constructors

        public Coupon()
        {
        }

        #endregion


        #region Public Properties

        // Return null object
        public static Coupon Null
        {
            get { return NullCoupon.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string CouponID { get { return _couponID; } set { _couponID = value; } }
        public DiscountTypeEnum DiscountType { get { return _discountType; } set { _discountType = value; } }
        public decimal DiscountAmount { get { return _discountAmount; } set { _discountAmount = value; } }
        public bool RepeatDiscount { get { return _repeatDiscount; } set { _repeatDiscount = value; } }
        public int MinimumQuantity { get { return _minimumQuantity; } set { _minimumQuantity = value; } }
        public int PromotionQuantity { get { return _promotionQuantity; } set { _promotionQuantity = value; } }
        public double Percentage { get { return _percentage; } set { _percentage = value; } }
        public ExpirationTypeEnum ExpirationType { get { return _expirationType; } set { _expirationType = value; } }
        public DateTime ExpirationDate { get { return _expirationDate; } set { _expirationDate = value; } }
        public int ExpirationQuantity
        {
            get { return _expirationQuantity; }
            set { _expirationQuantity = value; }
        }
        public int CurrentQuantity { get { return _currentQuantity; } set { _currentQuantity = value; } }
        public string MerchantNotes { get { return _merchantNotes; } set { _merchantNotes = value; } }
        public ProductFilterEnum ProductFilter { get { return _productFilter; } set { _productFilter = value; } }
        public string ProductIDs { get { return _productIDs; } set { _productIDs = value; } }
        public string CategoryIDs { get { return _categoryIDs; } set { _categoryIDs = value; } }
        public CustomerFilterEnum CustomerFilter { get { return _customerFilter; } set { _customerFilter = value; } }
        public string CustomerUserName { get { return _customerUserName; } set { _customerUserName = value; } }
        public decimal MinimumSubtotal { get { return _minimumSubtotal; } set { _minimumSubtotal = value; } }
        public ProductCostTypeEnum ProductCostType { get { return _productCostType; } set { _productCostType = value; } }
        public FreeShippingTypeEnum FreeShippingType { get { return _freeShippingType; } set { _freeShippingType = value; } }

        public ExpirationStatusEnum ExpirationStatus
        {
            get
            {
                switch (ExpirationType)
                {
                    case ExpirationTypeEnum.Date:
                        return GetExpirationStatusByDate();

                    case ExpirationTypeEnum.Quantity:
                        return GetExpirationStatusByQuantity();

                    case ExpirationTypeEnum.Both:
                        ExpirationStatusEnum status = GetExpirationStatusByDate();
                        if (status != ExpirationStatusEnum.NotExpired)
                            return status;
                        else
                            return GetExpirationStatusByQuantity();
                }

                return ExpirationStatusEnum.NotExpired;
            }
        }

        private bool IsContainPromotion( CartItemGroup cart )
        {
            foreach (ICartItem item in cart.CartItems)
            {
                if (item.IsPromotion)
                {
                    return true;
                }
            }
            return false;
        }

        private bool IsAnyProductDiscountable( CartItemGroup cart )
        {
            if (DiscountType == DiscountTypeEnum.FreeShipping)
            {
                if (FreeShippingType == FreeShippingTypeEnum.MatchingItemsOnly)
                {
                    foreach (ICartItem item in cart.CartItems)
                    {
                        if (!IsProductDiscountable( item.Product ))
                        {
                            return false;
                        }
                    }

                    return true;
                }
                else
                {
                    foreach (ICartItem item in cart.CartItems)
                    {
                        if (IsProductDiscountable( item.Product ))
                        {
                            return true;
                        }
                    }

                    return false;
                }
            }
            else
            {
                foreach (ICartItem item in cart.CartItems)
                {
                    if (IsProductDiscountable( item.Product ))
                    {
                        return true;
                    }
                }

                return false;
            }
        }

        private bool OrderAmountQualified(
            CartItemGroup cartItems,
            Customer customer )
        {
            decimal subTotal = 0;
            foreach (ICartItem item in cartItems.GetCartItemArray())
            {
                if (ProductCostType == Coupon.ProductCostTypeEnum.CouponEligibleProducts)
                {
                    if (VerifyCoupon( item.Product, customer ))
                    {
                        subTotal += item.GetSubtotal( customer.GetWholesaleStatus() );
                    }
                }
                else
                {
                    subTotal += item.GetSubtotal( customer.GetWholesaleStatus() );
                }
            }

            if (subTotal >= MinimumSubtotal)
                return true;
            else
                return false;
        }

        private decimal GetDiscount(
            CartItemGroup cartItems,
            Customer customer,
            VerifyCouponCallback verifyCouponCallback,
            out IList<decimal> discountLines )
        {
            if (String.IsNullOrEmpty( _couponID )
                || !OrderAmountQualified( cartItems, customer ))
            {
                //discountLines = originalDiscountLines;

                discountLines = new List<decimal>();
                for (int i = 0; i < cartItems.CartItems.Count; i++)
                    discountLines.Add( 0 );

                return 0;
            }
            else
                return GetDiscountPerOrder( cartItems.GetCartItemArray(), customer, verifyCouponCallback, out discountLines );
        }

        private bool IsDiscountableProduct( Product product )
        {
            return product.DiscountGroupID != "0";
        }

        #endregion


        #region public Methods

        public IList<Product> GetApplicableProducts( Culture culture )
        {
            IList<Product> list = new List<Product>();

            if (ProductFilter == Coupon.ProductFilterEnum.ByProductIDs)
            {
                string[] productIDsList = ProductIDs.Split( ',' );
                for (int i = 0; i < productIDsList.Length; i++)
                {
                    Product product = DataAccessContext.ProductRepository.GetOne(
                        culture, productIDsList[i], Stores.NullStore.Null.StoreID );

                    if (!product.IsNull
                        && !IsDiscountableProduct( product ))
                    {
                        list.Add( product );
                    }
                }
            }

            return list;
        }

        public IList<Category> GetApplicableCategories( Culture culture )
        {
            IList<Category> list = new List<Category>();

            if (ProductFilter == Coupon.ProductFilterEnum.ByCategoryIDs)
            {
                string[] categoryIDsList = CategoryIDs.Split( ',' );
                for (int i = 0; i < categoryIDsList.Length; i++)
                {
                    Category category =
                        DataAccessContext.CategoryRepository.GetOne( culture, categoryIDsList[i] );

                    if (!category.IsNull)
                        list.Add( category );
                }
            }

            return list;
        }

        public virtual ValidationStatus Validate(
            CartItemGroup cartItemGroup,
            Customer customer )
        {
            if (ExpirationStatus != ExpirationStatusEnum.NotExpired)
                return ValidationStatus.Expired;
            if (!IsCustomerDiscountable( customer ))
                return ValidationStatus.InvalidCustomer;
            if ((!IsAnyProductDiscountable( cartItemGroup )))
                return ValidationStatus.InvalidProduct;
            if (!OrderAmountQualified( cartItemGroup, customer ))
                return ValidationStatus.BelowMinimumOrderAmount;
            if (!QualifyQuantity( cartItemGroup ))
                return ValidationStatus.BelowMinimumQuantity;
            else
                return ValidationStatus.OK;
        }

        public bool IsProductDiscountable( Product product )
        {
            if (ProductFilter == Coupon.ProductFilterEnum.All)
            {
                return true;
            }
            else if (ProductFilter == Coupon.ProductFilterEnum.ByProductIDs)
            {
                if (product.ProductID == "0")
                {
                    return false;
                }
                return CheckExistInList( ProductIDs, product.ProductID );
            }
            else if (ProductFilter == Coupon.ProductFilterEnum.ByCategoryIDs)
            {
                if (product.ProductID == "0")
                {
                    return false;
                }

                ArrayList categoryIDsList = new ArrayList();

                String[] categoryIDs = AddParentCategoriesToList( product.CategoryIDs );
                return CheckExistInList( CategoryIDs, categoryIDs );
            }
            return false;
        }

        public bool IsCustomerDiscountable( Customer customer )
        {
            if (CustomerFilter == Coupon.CustomerFilterEnum.All)
                return true;
            else if (CustomerFilter == Coupon.CustomerFilterEnum.ByUserNames)
                return CheckExistInList( CustomerUserName, customer.UserName );
            else
                return false;
        }

        public virtual decimal GetDiscount(
            CartItemGroup cartItems,
            Customer customer,
            out IList<decimal> discountLines )
        {
            return GetDiscount( cartItems, customer, VerifyCoupon, out discountLines );
        }

        public virtual decimal GetDiscountNoExpiration(
            CartItemGroup cartItems,
            Customer customer,
            out IList<decimal> discountLines )
        {
            return GetDiscount( cartItems, customer, VerifyCouponNoExpiration, out discountLines );
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            Coupon newObject = (Coupon) MemberwiseClone();
            return newObject;
        }

        #endregion
    }
}
